"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var os = require("os");
var cmn = require("@akashic/akashic-cli-commons");
var archiver = require("archiver");
var readdir = require("fs-readdir-recursive");
var convert_1 = require("./convert");
function _completeExportZipParameterObject(param) {
    return {
        bundle: !!param.bundle,
        babel: !!param.babel,
        minify: !!param.minify,
        strip: !!param.strip,
        source: param.source || process.cwd(),
        dest: param.dest || "./game.zip",
        force: !!param.force,
        logger: param.logger || new cmn.ConsoleLogger(),
        hashLength: param.hashLength,
        omitEmptyJs: param.omitEmptyJs
    };
}
exports._completeExportZipParameterObject = _completeExportZipParameterObject;
// TODO akashic-cli-commons に移して export html と実装を共有する
function _checkDestinationValidity(dest, force) {
    return new Promise(function (resolve, reject) {
        fs.stat(path.resolve(dest), function (error, stat) {
            if (error) {
                if (error.code === "ENOENT") {
                    resolve();
                }
                else {
                    reject(new Error("Error code " + error.code + " for " + dest));
                }
                return;
            }
            if (!force)
                reject(new Error(dest + " already exists. Use --force option to overwrite."));
            else
                resolve();
        });
    });
}
exports._checkDestinationValidity = _checkDestinationValidity;
function promiseExportZip(param) {
    param = _completeExportZipParameterObject(param);
    var outZip = /\.zip$/.test(param.dest);
    var destDir = outZip ? fs.mkdtempSync(path.join(os.tmpdir(), "akashic-export-zip-")) : param.dest;
    return _checkDestinationValidity(param.dest, param.force)
        .then(function () {
        return convert_1.convertGame({
            bundle: param.bundle,
            babel: param.babel,
            minify: param.minify,
            strip: param.strip,
            source: param.source,
            dest: destDir,
            hashLength: param.hashLength,
            omitEmptyJs: param.omitEmptyJs,
            logger: param.logger
        });
    })
        .then(function () {
        if (!outZip)
            return;
        return new Promise(function (resolve, reject) {
            var files = readdir(destDir).map(function (p) { return ({
                src: path.resolve(destDir, p),
                entryName: p
            }); });
            var ostream = fs.createWriteStream(param.dest);
            var archive = archiver("zip");
            ostream.on("close", function () { return resolve(); });
            archive.on("error", function (err) { return reject(err); });
            archive.pipe(ostream);
            files.forEach(function (f) { return archive.file(f.src, { name: f.entryName }); });
            archive.finalize();
        });
        // TODO mkdtempのフォルダを削除すべき？
    });
}
exports.promiseExportZip = promiseExportZip;
function exportZip(param, callback) {
    promiseExportZip(param).then(function () { return callback(); }, callback);
}
exports.exportZip = exportZip;
